package com.oferteo.api.v1.client.rest;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Locale;

import org.apache.http.HttpEntity;
import org.apache.http.HttpStatus;
import org.apache.http.StatusLine;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;
import org.apache.log4j.Logger;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

/**
 * OferteoAPi
 * 
 *  Klasa umozliwia komunikacje z serwisem Oferteo
 *  
 */
public class OferteoApi
{

	private static final String HTTP_HEADER_ACCEPT_ENCODING_GZIP = "gzip";

	private static final Logger log = Logger.getLogger(OferteoApi.class);

	private static final String JSON_NAMES_OC = "oc";
	private static final String JSON_NAMES_OFERTEO_PREFIX_URL = "oferteoPrefixUrl";
	private static final String JSON_NAMES_SELECTED_LOCATION_NAME = "selectedLocationName";
	private static final String JSON_NAMES_SELECTED_CATEGORY_NAME = "selectedCategoryName";
	private static final String JSON_NAMES_RFPS_COUNTER = "rfpsCounter";
	private static final String JSON_NAMES_RFPS = "rfps";
	private static final String JSON_NAMES_SUBCATEGORIES = "subcategories";
	private static final String JSON_NAMES_LOCATIONS = "locations";

	public static final String CSS_MAIN = "main_css";
	
	
	private static final String HTTP_HEADER_ACCEPT_ENCODING = "Accept-Encoding";
	public static final String HTTP_HEADER_OFERTEO_API_KEY = "X-Oferteo-ApiKey";
	public static final int 	DEFAULT_SORT	= 0;
	private static final int MAX_RESULT_BYTES = 1024 * 1024 * 100;	//10 MB
	
	private static final String JSON_POSTFIX 	= ".json";
	private static final String LOCATIONS_URL 	= "locations";
	private static final String CATEGORIES_URL 	= "categories";
	private static final String RFPS_URL 		= "rfps";

	private static final String API_DESC_URL = "api-desc";
	
	private final OferteoApiData apiData;
	
	private String 	hostUrl;
	private String 	apiKey;

	private Integer selectedCategory;
	private String selectedLocation;
	private String widgetId;
	
	private String selectedCategoryUrl;
	private String selectedLocationUrl;
	private long selectedRfpId = -1;
	
	private String 	urlPrefix;
	private String 	urlCurrent;
	private String 	urlQueryString;
	
	
	private Locale locale = new Locale("pl", "PL");
	private int page;
	private int pageSize = 20;
	private int sort;
	private RfpRest rfpSelected;
	private RfpSearchResult rfpSearchResult;
	
	private HtmlType htmlType = HtmlType.STANDARD;
	
	
	/**
	 * Przyklady integracji: http://www.oferteo.pl/twoj-dzial-zlecen/przyklady-integracji
	 * 
	 * @param hostUrl - adres serwera Oferteo, np.: "http://api.oferteo.pl/v1/" 
	 * @param apiKey - klucz autoryzacyjny otrzymany od Oferteo
	 * @param urlPrefix - kontener w ramach ktorego bedzie prezentowana lista zlecen, np. "/zlecenia" 
	 */
	public OferteoApi(String hostUrl, String apiKey, String urlPrefix) 
	{
		this.hostUrl = hostUrl;
		this.apiKey = apiKey;
		this.urlPrefix = urlPrefix;
		this.apiData = OferteoApiData.getInstance(this);
	}

	
	ApiDescRest getApiDesc() {
		JSONObject apiDescJSONObject = getJsonDataObject(buildIndexUrl(API_DESC_URL));
		
		ApiDescRest desc = null;
		if (apiDescJSONObject != null) {
			//lista wynikow
			desc = new ApiDescRest(apiDescJSONObject);
		}
		return desc;
	}
	
	private void updateCurrentUrl(String newUrl) 
	{
		
		updateSelected(
				newUrl,
				ApiClientUtils.findCategoryInUrl(newUrl, urlPrefix, apiData.getCategoriesMap()),
				ApiClientUtils.findLocationInUrl(newUrl, urlPrefix, apiData.getLocationsMap()),
				ApiClientUtils.findRfpIdInUrl(newUrl),
				ApiClientUtils.findPagerInUrl(newUrl)
				);
	}
	
	private void updateSelected(String urlCurrent, String categoryUrl, String locationUrl, long rfpId, int page) 
	{
		this.urlCurrent = urlCurrent;
		this.selectedCategoryUrl = categoryUrl;
		this.selectedLocationUrl = locationUrl;
		this.selectedRfpId  = rfpId;
		this.page = page;
		
		//ustawienie wybranej kategorii i lokalizacji na podstawie danych z url-a
		if (selectedCategoryUrl != null) {
			this.selectedCategory = apiData.getCategoriesMap().get(selectedCategoryUrl);
		} else {
			this.selectedCategory = null;
		}
		if (selectedLocationUrl != null) {
			this.selectedLocation = apiData.getLocationsMap().get(selectedLocationUrl);
		} else {
			selectedLocation = null;
		}
	}

	public Integer getSelectedCategory()
	{
		return selectedCategory;
	}
	
	/**
	 * Ustawia maksymalna liczbe wynikow na stronie z lista zlecen. Domyslnie wartosc ustawiona na 20.
	 * @param pageSize - liczba elementow pokazywanych na jednej stronie
	 */
	public void setPageSize(int pageSize) 
	{
		if (pageSize > 0) {
			this.pageSize = pageSize;
		}
	}


	/**
	 * Aktualizacja URL, na podstawie ktorego odczytywane sa parametry zapytan REST.
	 * Dla przegladania listy zlecen URL moze sie skladac z nastepujacych elementow:<br>
	 * <code>prefix[/url_kategorii][/url_lokalizacji[/pager]</code>
	 * <ul>
	 * 	<li><strong>prefix</strong> - prefix URL-a, bedacy kontenerem, w ramach ktorego beda generowane linki, np. /zlecenia </li>
	 * 	<li><strong>url_kategorii</strong> - identyfikator kategorii w formacie "user-friendly URL" </li>
	 * 	<li><strong>url_lokalizacji</strong> - identyfikator wybranej lokalizacji w formacie "user-friendly URL"</li>
	 * 	<li><strong>pager</strong> - numer strony z wynikami (liczba 1...)
	 * </ul>
	 * Przyklady:<br>
	 * <ul>
	 * 	<li>/zlecenia/dolnoslaskie</li>
	 * 	<li>/zlecenia/budownictwo-uslugi</li>
	 * 	<li>/zlecenia/budownictwo-uslugi/dolnoslaskie</li>
	 * 	<li>/zlecenia/budownictwo-uslugi/dolnoslaskie/2</li>
	 * </ul>
	 * 
	 * Dla podgladu pojedynczego zlecenia URL moze sie skladac z nastepujacych elementow:<br>
	 * <code>prefix/rfp_id</code>
	 * <ul>
	 * 	<li><strong>prefix</strong> - prefix URL-a, bedacy kontenerem, w ramach ktorego beda generowane linki, np. /zlecenia </li>
	 * 	<li><strong>rfp_id</strong> - identyfikator zlecenia w formacie "user-friendly URL" </li>
	 * </ul>
	 * Przyklad:<br>
	 * <ul>
	 * 	<li>/zlecenia/zlece-wykonanie-elewacji-na-domu-pow-ok-200m2-012345</li>
	 * </ul>
	 * <br>
	 * URL-e w takiej postaci sa generowane przez OferteoApi w metodach budujacych przykladowy html.
	 * Przyklady integracji: http://www.oferteo.pl/twoj-dzial-zlecen/przyklady-integracji
	 * @param urlCurrent
	 */
	public void setUrlCurrent(String urlCurrent) 
	{
		updateCurrentUrl(urlCurrent);	
	}
	
	/**
	 * Ustawia locale wg ktorego beda generowane komunikaty. 
	 * Domyslnie ustawione na pl_PL
	 * @param locale
	 */
	public void setLocale(Locale locale) 
	{
		this.locale = locale;
	}
	
	public String getJsonData(String jsonUrl) throws ClientProtocolException, IOException
	{
		String data = null;
		HttpGet httpget = new HttpGet(jsonUrl);
		httpget.addHeader(HTTP_HEADER_OFERTEO_API_KEY, this.apiKey);
		httpget.addHeader(HTTP_HEADER_ACCEPT_ENCODING, HTTP_HEADER_ACCEPT_ENCODING_GZIP);
		
		CloseableHttpClient httpclient = HttpClients.createDefault();
		CloseableHttpResponse response = httpclient.execute(httpget);
		StatusLine responseStatus = response.getStatusLine();
		if (responseStatus.getStatusCode() == HttpStatus.SC_OK) {
			try {
				HttpEntity entity = response.getEntity();
				if (entity == null) {
					log.warn("Empty response for url: " + jsonUrl);
					return null;
				} else if (entity.getContentLength() > MAX_RESULT_BYTES) {
					log.warn("Response for url: " + jsonUrl + " is larger than " + MAX_RESULT_BYTES + " bytes. Returning null");
				}
				data = EntityUtils.toString(entity);
			} finally {		
				response.close();
			}
		} else {
			int status = responseStatus.getStatusCode();
			String reasonPhrase = responseStatus.getReasonPhrase();
			log.warn("Incorrect response status code for url: " + jsonUrl + "(API key: " + this.apiKey + ")\n\t Status code: " + status + "\n\t Reason phrase: " + reasonPhrase);
		}

		return data;
	}

	/**
	 * Wywoluje serwis REST w celu pobrania listy elementow
	 * @param jsonUrl URL do serwisu REST, zwracajacego dane w formacje JSON
	 * @return null jesli wystapil blad podczas wywolania serwisu, w przeciwnym wypadku
	 * lista elementow pobranych z serwera, skonwertowana do typu JSONArray
	 */
	public JSONArray getJsonDataArray(String jsonUrl) 
	{
		String data;
		try {
			data = getJsonData(jsonUrl);
			if (data == null) {
				return null;
			}
		} catch (ClientProtocolException e) {
			log.error("Error while getting JSON data, url: " + jsonUrl, e);
			return null;
		} catch (IOException e) {
			log.error("Error while getting JSON data, url: " + jsonUrl, e);
			return null;
		}
		
		JSONArray returnArray;
		try {
			returnArray = new JSONArray(data);
		} catch (JSONException e) {
			log.error("Could not convert do JSONArray, url: " + jsonUrl, e);
			returnArray = null;
		}
		return returnArray;
	}

	/**
	 * Wywoluje serwis REST w celu pobrania danych i konwertuje do postaci obiektu JSON
	 * @param jsonUrl URL do serwisu REST, zwracajacego dane w formacje JSON
	 * @return null jesli wystapil blad podczas wywolania serwisu, w przeciwnym wypadku
	 * lista elementow pobranych z serwera
	 */
	public JSONObject getJsonDataObject(String jsonUrl)
	{
		String data;
		try {
			data = getJsonData(jsonUrl);
			if (data == null) {
				return null;
			}
		} catch (ClientProtocolException e) {
			log.error("Error while getting JSON data, url: " + jsonUrl, e);
			return null;
		} catch (IOException e)	{
			log.error("Error while getting JSON data, url: " + jsonUrl, e);
			return null;
		}

		JSONObject jsonObject;
		try {
			jsonObject = new JSONObject(data);
		} catch (JSONException e) {
			log.error("Could not convert do JSONObject, url: " + jsonUrl, e);
			jsonObject = null;
		}
		return jsonObject;
	}
	
	private String buildShowUrl(String restUrl, String selectedId) 
	{
		return this.hostUrl + restUrl + "/" + selectedId + JSON_POSTFIX;
	}
	
	private String buildIndexUrl(String restUrl) 
	{
		return this.hostUrl + restUrl + JSON_POSTFIX 
				+ "?categoryId=" + (this.selectedCategory != null ? this.selectedCategory : "") 
				+ "&locationId=" + (this.selectedLocation != null ? this.selectedLocation : "")
				+ "&pageNumber=" + (this.page > 0 ? this.page : 1)
				+ "&pageSize=" + this.pageSize
				+ "&sort=" + this.sort
				;
	}
	
	
	/**
	 * Wywoluje serwis Oferteo w celu pobrania danych wybranej lokalizacji
	 * @param locationId
	 * @return
	 */
	public LocationRest getLocation(String locationId) 
	{
		JSONObject locationJSON = getJsonDataObject(buildShowUrl(LOCATIONS_URL, locationId));
		if (locationJSON != null) {
			return fillLocationTree(locationJSON);
		}
		return null;
	}
	
	/**
	 * Wywoluje serwis Oferteo w celu pobrania dostepnych lokalizacji.
	 * Parametry pomocnicze ustawione na podstawie urlCurrent (szczegoly: {@link #setUrlCurrent(String)})
	 * @return
	 */
	public Collection<LocationRest> getLocationsTree() 
	{
		Collection<LocationRest> resultList = new ArrayList<LocationRest>();
		JSONArray locationsJSON = getJsonDataArray(buildIndexUrl(LOCATIONS_URL));
		if (locationsJSON != null && locationsJSON.length() > 0) {
			for (int i = 0; i< locationsJSON.length(); i++) {
				JSONObject firstLevelLocationJSON = locationsJSON.getJSONObject(i);
				resultList.add(fillLocationTree(firstLevelLocationJSON));
			}
		}
		return resultList;
	}

	private LocationRest fillLocationTree(JSONObject locationJSON) 
	{
		LocationRest locationRest = new LocationRest(locationJSON);
		//find secondLevelLocations
		JSONArray nextLevelLocationsJSONArray = locationJSON.getJSONArray(JSON_NAMES_LOCATIONS);
		if (nextLevelLocationsJSONArray != null && nextLevelLocationsJSONArray.length() > 0) {
			for (int j = 0; j < nextLevelLocationsJSONArray.length(); j++) {
				JSONObject nextLevelLocationJSON = nextLevelLocationsJSONArray.getJSONObject(j);
				LocationRest nextLevelLocation = fillLocationTree(nextLevelLocationJSON);
				nextLevelLocation.setParent(locationRest);
				locationRest.getLocations().add(nextLevelLocation);
			}
		}
		return locationRest;
	}
	
	/**
	 * Wywoluje serwis Oferteo w celu pobrania informacji o wybranej kategorii
	 * @param categoryId
	 * @return
	 */
	public CategoryRest getCategory(String categoryId) 
	{
		JSONObject categoryJSON = getJsonDataObject(buildShowUrl(CATEGORIES_URL, categoryId));
		if (categoryJSON != null) {
			return fillCategoriesTree(categoryJSON);
		}
		return null;
	}

	/**
	 * Wywoluje serwis Oferteo w celu pobrania dostepnych kategorii.
	 * Parametry pomocnicze ustawione na podstawie urlCurrent (szczegoly: {@link #setUrlCurrent(String)})
	 * @return
	 */
	public Collection<CategoryRest> getCategoriesTree() 
	{
		Collection<CategoryRest> resultList = new ArrayList<CategoryRest>();
		JSONArray categoriesJSON = getJsonDataArray(buildIndexUrl(CATEGORIES_URL));
		if (categoriesJSON != null && categoriesJSON.length() > 0) {
			for (int i = 0; i< categoriesJSON.length(); i++) {
				JSONObject categoryJSON = categoriesJSON.getJSONObject(i);
				CategoryRest subcategory = fillCategoriesTree(categoryJSON);
				resultList.add(subcategory);
			}
		}
		return resultList;
	}

	private CategoryRest fillCategoriesTree(JSONObject categoryJSON) 
	{
		CategoryRest categoryRest = new CategoryRest(categoryJSON);
		//find secondLevelLocations
		JSONArray subcategoriesJSONArray = categoryJSON.getJSONArray(JSON_NAMES_SUBCATEGORIES);
		if (subcategoriesJSONArray != null && subcategoriesJSONArray.length() > 0) {
			for (int j = 0; j < subcategoriesJSONArray.length(); j++) {
				JSONObject subcategoryJSON = subcategoriesJSONArray.getJSONObject(j);
				CategoryRest subcategoryRest = fillCategoriesTree(subcategoryJSON);
				subcategoryRest.setParent(categoryRest);
				categoryRest.getSubcategories().add(subcategoryRest);
			}
		}
		return categoryRest;
	}

	public boolean showingSelectedRfp() {
		return this.selectedRfpId > 0;
	}
	
	
	/**
	 * Ustawia parametry dla zapytan na podstawie wybranego RFP:
	 * <ul>
	 * 	<li>categoryUrl - na podstawie kategorii wybranego RFP</li>
	 * 	<li>voivodShipUrl - na podstawie wojewodztwa wybranego RFP</li>
	 * </ul>
	 * Dzieki temu nastepne zapytania po kategorie i lokalizacje beda mialy 
	 * ustawione parametry ustawione na podstawie aktualnego zlecenia 
	 * @param rfp - wybrany RFP do prezentowania szczegolow
	 */
	public void updateSelectedCategoryAndLocation(RfpRest rfp) {
		if (rfp == null) {
			return;
		}
		String voivodShipUrl = rfp.getDstVoivodshipUrl();
		
		String categoryUrl = rfp.getCategoryUrl();
		updateSelected(
				this.urlCurrent, 
				categoryUrl, 
				voivodShipUrl, 
				this.selectedRfpId, 
				this.page
				);
	}
	
	/**
	 * Pobiera za pomoca zapytania REST szczegoly zapytania wybranego z listy.<br>
	 * Identyfikator zapytania jest odczytany z biezacego URL (patrz: {@link #setUrlCurrent(String)})
	 * @return
	 */
	public RfpRest getRfp() {
		
		this.rfpSelected = getRfp(this.selectedRfpId);
		
		return rfpSelected;
	}
	
	/**
	 * Pobiera za pomoca zapytania REST szczegoly zapytania o podanym id
	 * @param idRfp
	 * @return null jesli <code>idRfp &lt; 0 </code> lub jesli nie udalo sie wczytac danych RFP
	 */
	public RfpRest getRfp(long idRfp) 
	{
		if (idRfp < 0) {
			return null;
		}
		RfpRest result = null;
		JSONObject rfpJSONObject = getJsonDataObject(buildShowUrl(RFPS_URL, "" + idRfp));
		if (rfpJSONObject != null) {
			result = new RfpRest(rfpJSONObject, locale);
		}
		return result;
	}

	/**
	 * Wykonuje zapytanie REST w celu pobrania listy zapytan (RFP).
	 * Parametry zapytania sa ustawione na podstawie aktualnego URL-a (patrz: {@link #setUrlCurrent(String)})
	 * @return
	 */
	public RfpSearchResult getRfps() 
	{
		List<RfpRest> resultList = new ArrayList<RfpRest>();
		JSONObject rfpJSONObject = getJsonDataObject(buildIndexUrl(RFPS_URL));

		RfpSearchResult searchResult = null;
		if (rfpJSONObject != null) {
			//lista wynikow
			JSONArray rfpJSONArray = rfpJSONObject.getJSONArray(JSON_NAMES_RFPS); 
			if (rfpJSONArray != null && rfpJSONArray.length() > 0) {
				for (int i = 0; i< rfpJSONArray.length(); i++) {
					JSONObject rfpJSON = rfpJSONArray.getJSONObject(i);
					resultList.add(new RfpRest(rfpJSON, locale));
				}
			}
			searchResult = new RfpSearchResult(
					resultList,
					rfpJSONObject.getInt(JSON_NAMES_RFPS_COUNTER),
					rfpJSONObject.getString(JSON_NAMES_SELECTED_CATEGORY_NAME),
					rfpJSONObject.getString(JSON_NAMES_SELECTED_LOCATION_NAME),
					rfpJSONObject.getString(JSON_NAMES_OFERTEO_PREFIX_URL),
					rfpJSONObject.getString(JSON_NAMES_OC)
					);
		} else {
			//jesli nie udalo sie pobrac danych z serwera to ustaw parametry domyslne
			searchResult = new RfpSearchResult(
					resultList,
					0,
					"",
					"",
					"",
					null
					);
		}
		this.rfpSearchResult = searchResult;
		return searchResult;
	}

	/**
	 * Na podstawie podanych kategorii buduje HTML przedstawiajacy kategorie w postaci listy zbudowanej z elementow {@literal <ul> oraz <li>}.
	 * @param categories - kolekcja kategorii najwyzszego poziomu
	 * @return
	 */
	public String getCategoryHtml(Collection<CategoryRest> categories) 
	{
		CategoriesHtmlBuilder htmlBuilder = getCategoriesHtmlBuilder(categories, this.urlPrefix, selectedLocationUrl);
		return htmlBuilder.getHtml();
	}
	
	
	/**
	 * Na podstawie podanych lokalizacji buduje HTML przedstawiajacy lokalizacje w postaci listy zbudowanej z elementow {@literal <ul> oraz <li>}.
	 * @param locations - kolekcja lokalizacji najwyzszego poziomu
	 * @return
	 */
	public String getLocationHtml(Collection<LocationRest> locations) 
	{
		String cityName = null;
		String poviatName = null;
		String poviatUrl = null;
		if (showingSelectedRfp()) {
			cityName = this.rfpSelected.getDstCity();
			if (rfpSelected.getDstPoviatUrl() != null && rfpSelected.getDstPoviatUrl().length() > 0 && this.apiData.getLocationsMap().containsKey(rfpSelected.getDstPoviatUrl())) {
				poviatUrl = rfpSelected.getDstPoviatUrl();
				poviatName = rfpSelected.getDstPoviat();
				
			}
		}
		
		LocationsHtmlBuilder htmlBuilder = getLocationsHtmlBuilder(locations, this.urlPrefix, selectedCategoryUrl, showingSelectedRfp(), cityName, poviatName, poviatUrl);
		return htmlBuilder.getHtml();
	}

	/**
	 * Na podstawie obiektu reprezentujacego pojedyncze zapytanie buduje HTML prezentujacy szczegoly zapytania
	 * @param apiRfp
	 * @return
	 */
	public String getRfpHtml(RfpRest apiRfp) 
	{
		RfpHtmlBuilder htmlBuilder = new RfpHtmlBuilder(apiRfp, urlPrefix, locale, widgetId);
		return htmlBuilder.getHtml();
	}
	
	
	public String getRfpHtmlAll(RfpRest apiRfp, Collection<CategoryRest> categories, Collection<LocationRest> locations) {
		String cityName = null;
		String poviatName = null;
		String poviatUrl = null;
		if (showingSelectedRfp() && apiRfp != null) {
			cityName = this.rfpSelected.getDstCity();
			if (rfpSelected.getDstPoviatUrl() != null && rfpSelected.getDstPoviatUrl().length() > 0 && apiData.getLocationsMap().containsKey(rfpSelected.getDstPoviatUrl())) {
				poviatUrl = rfpSelected.getDstPoviatUrl();
				poviatName = rfpSelected.getDstPoviat();
				
			}
		}
		CategoriesHtmlBuilder categoriesBuilder = getCategoriesHtmlBuilder(categories, this.urlPrefix, selectedLocationUrl);
		LocationsHtmlBuilder locationsBuilder =  getLocationsHtmlBuilder(locations, this.urlPrefix, selectedCategoryUrl, showingSelectedRfp(), cityName, poviatName, poviatUrl);
		RfpHtmlBuilder htmlBuilder = new RfpSingleColumnHtmlBuilder(apiRfp, categoriesBuilder, locationsBuilder, urlPrefix, locale, widgetId);
		return htmlBuilder.getHtml();
	}
	
	/**
	 * Na podstawie obiektu z wynikami wyszukiwania zapytan ofertowych buduje HTML przedstawiacy dane w postaci listy
	 * podzielonej na zestawy po 20 elementow.
	 * @param rfpSearchResult - obiekt z wynikami wyszukiwania ({@link #getRfps()}).
	 * @return
	 */
	public String getRfpsListHtml(RfpSearchResult rfpSearchResult) 
	{
		return getRfpsListHtml(rfpSearchResult, 20);
	}
	
	/**
	 * Na podstawie obiektu z wynikami wyszukiwania zapytan ofertowych buduje HTML przedstawiacy dane w postaci listy
	 * podzielonej na zestawy po <code>pageSize</code> elementow.
	 * @param rfpSearchResult - obiekt z wynikami wyszukiwania ({@link #getRfps()}).
	 * @param pageSize - liczba elementow na jednej stronie
	 * @return
	 */
	public String getRfpsListHtml(RfpSearchResult rfpSearchResult, int pageSize) 
	{
		RfpsHtmlBuilder htmlBuilder = getRfpsHtmlBuilder(rfpSearchResult, urlPrefix, locale, this.urlCurrent, this.selectedCategoryUrl, this.selectedLocationUrl, pageSize, this.sort, this.urlQueryString);
		return htmlBuilder.getHtml();
	}

	public String getRfpsListHeaderHtml(RfpSearchResult rfpSearchResult) 
	{
		RfpsHtmlBuilder htmlBuilder = getRfpsHtmlBuilder(rfpSearchResult, urlPrefix, locale, this.urlCurrent, this.selectedCategoryUrl, this.selectedLocationUrl, pageSize, this.sort, this.urlQueryString);
		return htmlBuilder.getHeaderHtml();
	}

	/**
	 * Ustawienie parametrow dodatkowych, ktore beda uzyte przy wywolywaniu serwisow REST, np:
	 * <ul>
	 * 	<li> sort - rodzaj sortowania
	 * </ul>
	 * @return
	 */
	public void setUrlQueryString(String urlQueryString) 
	{
		this.urlQueryString = urlQueryString;
		this.sort = ApiClientUtils.getSortFromUrl(urlQueryString);
	}

	public String getWidgetId() {
		return widgetId;
	}

	public void setWidgetId(String widgetId) {
		this.widgetId = widgetId;
	}
	
	private RfpsHtmlBuilder getRfpsHtmlBuilder(RfpSearchResult rfpSearch, String urlPrefix, Locale locale, String currentUrl, String selectedCategoryUrl, String selectedLocationUrl, int pageSize, int sort, String urlQueryString) {
		if (HtmlType.SINGLE_COLUMN.equals(this.htmlType)) {
			return new RfpsSingleColumnHtmlBuilder(rfpSearch, urlPrefix, locale, currentUrl, selectedCategoryUrl, selectedLocationUrl, pageSize, DEFAULT_SORT, urlQueryString, this.widgetId);
		} else {
			return new RfpsHtmlBuilder(rfpSearch, urlPrefix, locale, currentUrl, selectedCategoryUrl, selectedLocationUrl, pageSize, DEFAULT_SORT, urlQueryString, this.widgetId);
		}
	}
	
	private LocationsHtmlBuilder getLocationsHtmlBuilder(Collection<LocationRest> locationsTree, String urlPrefix, String categorySelected, boolean showingRfp, String selectedCityName, String selectedPoviatName, String selectedPoviatUrl) {
		if (HtmlType.SINGLE_COLUMN.equals(this.htmlType)) {
			return new LocationsSingleColumnHtmlBuilder(locationsTree, urlPrefix, categorySelected, showingRfp, selectedCityName, selectedPoviatName, selectedPoviatUrl, locale);
		} else {
			return new LocationsHtmlBuilder(locationsTree, urlPrefix, categorySelected);
		}
	}
	
	private CategoriesHtmlBuilder getCategoriesHtmlBuilder(Collection<CategoryRest> categoriesTree, String urlPrefix, String locationSelected) {
		if (HtmlType.SINGLE_COLUMN.equals(this.htmlType)) {
			return new CategoriesSingleColumnHtmlBuilder(categoriesTree, urlPrefix, locationSelected, locale, showingSelectedRfp());
		} else {
			return new CategoriesHtmlBuilder(categoriesTree, urlPrefix, locationSelected);
		}
	}
	
	public HtmlType getHtmlType()
	{
		return htmlType;
	}

	public void setHtmlType(HtmlType htmlType)
	{
		this.htmlType = htmlType;
	}
	

	public String getMetaTags() {
		if (rfpSearchResult != null) {
			return ApiClientUtils.getPagerMetaTags(this.urlCurrent, this.rfpSearchResult.getRfpsCounter(), this.pageSize);
		} else {
			return "";
		}
		
	}
	
	public String getCssPath(String cssName) {
		return apiData.getCssPath(cssName); 
	}
	
	public String getJavascriptPath(String jsName) {
		return apiData.getJavascriptPath(jsName);
	}
	
	
}
