package com.oferteo.api.v1.client.rest;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import org.apache.log4j.Logger;


/**
 * Singleton do zapamietania wspolnej konfiguracji
 *
 */
public class OferteoApiData
{
	
	private static final Logger log = Logger.getLogger(OferteoApiData.class);
	private static final long API_REFRESH_INTERVAL = 60*60*1000;	//godzina
	private static final long API_REFRESH_ON_ERROR_INTERVAL = 5*60*1000;	//5 minut
	
	/** Mapa zawierajaca wszystkie dostepne kategorie categoryUrl -> categoryId */
	private final Map<String, Integer> categoryMap;
	
	/** Mapa zawierajaca wszystkie dostepne lokalizacje locationUrl -> locationId */
	private final Map<String, String> locationsMap;

	/** Dnae opisujace serwis*/
	private ApiDescRest apiDesc;

	private static OferteoApiData instance;
	
	private static final Object syncMonitor = new Object();
	
	private long nextRefreshTimestamp = -1;

	
	
	private OferteoApiData( Map<String, Integer> categoryMap, Map<String, String> locationsMap, ApiDescRest apiDesc) 
	{
		this.categoryMap = new ConcurrentHashMap<String, Integer>(categoryMap);
		this.locationsMap = new ConcurrentHashMap<String, String>(locationsMap);
		if (apiDesc != null) {
			this.apiDesc = apiDesc;
			this.nextRefreshTimestamp = System.currentTimeMillis() + API_REFRESH_INTERVAL;
		} else {
			this.nextRefreshTimestamp = System.currentTimeMillis() + API_REFRESH_ON_ERROR_INTERVAL;
		}
	}
	
	public static OferteoApiData getInstance(OferteoApi api) 
	{
		if (instance == null || System.currentTimeMillis() > instance.nextRefreshTimestamp) {
			synchronized (syncMonitor) {
				if (instance == null || System.currentTimeMillis() > instance.nextRefreshTimestamp) {
					try {
						Map<String, String> locations = fillLocationsMap(api.getLocationsTree());
						Map<String, Integer> categories = fillCategoriesMap(api.getCategoriesTree());
						ApiDescRest apiDesc = api.getApiDesc();
						instance = new OferteoApiData(categories, locations, apiDesc);
					} catch (Exception e) {
						
						
						log.error("Error initializing OferteoApiConfig class", e);
						if (instance != null) {
							instance.nextRefreshTimestamp = System.currentTimeMillis() +  API_REFRESH_ON_ERROR_INTERVAL;
						}
					}
				}
			}
		}
		return instance;
	}
	
	
	public Map<String, Integer> getCategoriesMap() {
		return this.categoryMap;
	}
	
	public Map<String, String> getLocationsMap() {
		return this.locationsMap;
	}
	
	
	private static Map<String, String> fillLocationsMap(Collection<LocationRest> locations) 
	{
		Map<String, String> newLocationsMap = new HashMap<String, String>();
		for (LocationRest loc : locations) {
			newLocationsMap.put(loc.getUrlLocation(), loc.getId());
			//add all existing sublocations
			newLocationsMap.putAll(fillLocationsMap(loc.getLocations()));
		}
		return newLocationsMap;
	}

	private static Map<String, Integer> fillCategoriesMap(Collection<CategoryRest> categories) 
	{
		Map<String, Integer> newCategoriesMap = new HashMap<String, Integer>();
		for (CategoryRest category: categories) {
			newCategoriesMap.put(category.getUrlCategory(), category.getId());
			//add all existing subcategories
			newCategoriesMap.putAll(fillCategoriesMap(category.getSubcategories()));
		}
		return newCategoriesMap;
	}
	
	public String getCssPath(String cssName) {
		if (cssName != null && this.apiDesc != null) {
			return this.apiDesc.getCssPaths().get(cssName);
		}
		return null;
	}
	
	public String getJavascriptPath(String jsName) {
		if (jsName != null && this.apiDesc != null) {
			return this.apiDesc.getJsPaths().get(jsName);
		}
		return null;
	}

}
